# ⭐️ graphql-automock ⭐️

**Automatically mock GraphQL schemas for better testing.**

Features:

- Automatically and deterministically mock GraphQL schemas
- Mock `react-apollo` for simple UI testing
- Control schema execution to reliably test loading and success states

## Getting started

Install via npm or yarn:

```sh
npm install --save-dev graphql-automock
yarn add --dev graphql-automock
```

## Mocking just the schema

Simply pass your GraphQL type definitions to `mockSchema` and
you're ready to go:

```javascript
import { mockSchema } from "graphql-automock";
import { graphql } from "graphql";

const types = `
  type Query {
    recentPosts: [Post!]!
  }

  type Post {
    id: ID!
    content: String!
    likes: Int!
  }
`;

const mocked = mockSchema(types);

const query = `{
  recentPosts {
    id
    content
    likes
  }
}`;

graphql(mocked, query);
```

Without any further configuration, this query will return:

```json
{
  "data": {
    "recentPosts": [
      {
        "id": "recentPosts.0.id",
        "content": "recentPosts.0.content",
        "likes": 2
      },
      {
        "id": "recentPosts.1.id",
        "content": "recentPosts.1.content",
        "likes": 2
      }
    ]
  }
}
```

To understand how these values are derived, see [Default values](#default-values).

## Mocking react-apollo

In addition to schema mocking, `<MockApolloProvider>` makes the testing of UI components much easier.

Simply wrap your elements in a `<MockApolloProvider>`, and any
`graphql()` and `<Query>` components in the tree will be able to function:

```javascript
import { MockApolloProvider } from "graphql-automock";

<MockApolloProvider schema={types}>
  <Post id="123" />
</MockApolloProvider>;
```

One issue to be aware of when doing this is that components will first enter a loading state, before the query resolves and components re-render
with the query result. You will most likely want to test both states. `SchemaController` allows you to step through schema execution to do just that:

```javascript
import { SchemaController, MockApolloProvider } from "graphql-automock";
import TestUtils from "react-dom/test-utils";

it("renders a Post", async () => {
  const controller = new SchemaController();
  const tree = TestUtils.renderIntoDocument(
    // Adding a controller pauses schema execution
    <MockApolloProvider schema={types} controller={controller}>
      <Post id="123" />
    </MockApolloProvider>
  );

  // Test loading state
  const spinners = TestUtils.scryRenderedComponentsWithType(tree, Spinner);
  expect(spinners).toHaveLength(1);

  // Allow schema to run, and wait for it to finish
  await controller.run();

  // Test success state
  const content = TestUtils.scryRenderedComponentsWithType(tree, Content);
  expect(content).toHaveLength(1);
});
```

## Customizing mocks

Automatically mocking the entire schema with sensible, deterministic data allows test code to customize _only the data that affects the test_. This results in test code that is more concise and easier to understand:

```javascript
// We're only interested in the behaviour of likes...
it("hides the likes count when there are no likes", () => {
  const mocks = {
    Post: () => ({
      likes: 0 // ...so we only customize that data
    })
  };

  // Using mockSchema
  const mockedSchema = mockSchema({
    schema: types,
    mocks: mocks
  });

  // Using MockApolloProvider
  const mockedElements = (
    <MockApolloProvider schema={types} mocks={mocks}>
      <Post id="123" />
    </MockApolloProvider>
  );

  // Continue with test...
});
```

## Default values

To ensure that tests are reliable, the values generated by graphql-automock
are 100% deterministic. The following default values are used:

- **Boolean**: `true`
- **Int**: `2`
- **Float**: `3.14`
- **String**: Path to value
- **ID**: Path to value
- **Enum**: The first enum value, sorted alphabetically by name
- **Interface**: The first possible implementation, sorted alphabetically by name
- **Union**: The first possible member type, sorted alphabetically by name
- **List length**: `2`

## API Reference

### mockSchema()

Create a mocked GraphQL schema.

```javascript
function mockSchema(schema: String | GraphQLSchema): GraphQLSchema;

function mockSchema({
  schema: String | GraphQLSchema,
  mocks: { [String]: MockResolverFn },
  controller: SchemaController
}): GraphQLSchema;
```

### mockApolloClient()

Create a mocked Apollo Client.

```javascript
function mockApolloClient(schema: String | GraphQLSchema): ApolloClient;

function mockApolloClient({
  schema: String | GraphQLSchema,
  mocks: { [String]: MockResolverFn },
  controller: SchemaController
}): ApolloClient;
```

### \<MockApolloProvider\>

React component that renders a mocked ApolloProvider.

```javascript
<MockApolloProvider
  schema={String | GraphQLSchema}
  mocks={{ [String]: MockResolverFn }}
  controller={SchemaController}
>
```

### type MockResolverFn

```javascript
type MockResolverFn = (parent, args, context, info) => any;
```

### SchemaController

#### pause()

Pause schema execution until it is explicitly resumed.
SchemaController starts in this state.

#### run()

Resume schema execution if it is paused. Returns a Promise that resolves when all pending GraphQL resolvers have finished executing.
If the schema was not paused, returns a resolved Promise.
